/* Josephus - a GUI Animation Example of Josephus Problem
 * Bob Wilson
 * 12/08/2017
 * 
 * */

import java.awt.*;
import javax.swing.*;
import java.util.ArrayList;

public class Josephus extends JFrame implements Runnable
{
  private int animationState = 0;
  
  public static void main(String[] args) 
  {
    // Get out of static context
    new Josephus().execute();
  }
  
  // Sailing constructor only needs to set title bar for its parent JFrame
  public Josephus()
  {
    super("Josephus");
  }
  
  // This is the rest of the main method - now out of static context
  private void execute()
  {
    // Set up the GUI Window
    setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);
    JPanel jp = new JPanel();
    
    ImageIcon soldier = new ImageIcon ("Soldier.gif");
    ArrayList<JLabel> labels = new ArrayList<JLabel>();
    for (int i = 0; i < 7; i++) {
      labels.add(new JLabel (Integer.toString(i + 1), soldier, SwingConstants.CENTER));
      jp.add(labels.get(i));
    }
    jp.setPreferredSize (new Dimension (1000, 200));
    add(jp);
    pack();
    setVisible(true);
    
    // Now start the animation state machine in a separate thread
    new Thread(this).start();
    
    // The current thread is not blocked, we are free to do other things
    // If we had just called run() directly, it would never return (infinite loop)
    System.out.println("Done.");
  }
  
  public void run() 
  {
    while (true) {                 // Loop until GUI frame is closed
      repaint();                      // Repaint the screen
      try {
        Thread.sleep(2000);           // Delay (in millisecs)
      }
      catch (Exception e) {
      }
      if (animationState < 8) {       // Update State Variable
        animationState++;
      }
      else {
        animationState = 0;
      }
    }
  }
  
  public void paint(Graphics screen) 
  {
    super.paint(screen);
    // cross out the dead soldiers one by one
    switch(animationState)
    {
      case 7:
        drawWinner(screen, 460, 50);
      case 6:
        drawX(screen, 140, 50);
      case 5:
        drawX(screen, 570, 50);
      case 4:
        drawX(screen, 780, 50);
      case 3:
        drawX(screen, 245, 50);
      case 2:
        drawX(screen, 675, 50);
      case 1:
        drawX(screen, 355, 50);
    }
  }
 
  private void drawX(Graphics screen, int w, int h)
  {
    screen.setColor(Color.BLACK);
    screen.drawLine(w, h, w + 90, h + 90);
    screen.drawLine(w, h + 1, w + 89, h + 90);
    screen.drawLine(w + 1, h, w + 90, h + 89);
    screen.drawLine(w + 90, h, w, h + 90);
    screen.drawLine(w + 90, h + 1, w + 1, h + 90);
    screen.drawLine(w + 89, h, w, h + 89);
  }
  
  private void drawWinner(Graphics screen, int w, int h)
  {
    screen.setColor(Color.RED);
    screen.setFont(new Font(null, Font.BOLD, 32));
    screen.drawRect(w, h, 90, 90);
    screen.drawString("The Winner!", 450, 200);
  }
}
